/*
  F-ZTAT ւ̏ݐ
  Satofumi KAMIMURA
  $Id$
*/

#include "stdafx.h"

#include "fztatCtrl.h"
#include "wctrl_dat.h"
#include "sformat.h"

//#define SINGLE_DEBUG


FZtatCtrl::FZtatCtrl(const char *sFormatFile, const char *serialPort)
  : sendSize(0), sfile(sFormatFile), port(serialPort), con(NULL), fd(NULL),
    line_cnt(1), buffer_size(0) {
  initBuffer();
}


FZtatCtrl::~FZtatCtrl(void) {
  if (con) {
    delete con;
  }
  if (fd) {
    fclose(fd);
  }
}


void FZtatCtrl::setupBaudrate(void) {
  
  /* ^[Qbg 0x00 M܂ 0x00 𑗐M */
  bool reply = false;
  int wait = 0;
  char ch = 0x00;
  do {
    con->send(&ch, 1);
    if ((con->recv(&ch, 1, SCI_TIMEOUT)) && (ch == 0x00)) {
      reply = true;
      break;			/* M */
    }
  } while ((wait += SCI_TIMEOUT) < SCI_TOTAL_TIMEOUT);
  if (!reply) {
    throw FZtatCtrlError("first connection: no reply 0x00...");
  }
  
  /* 0x55  1byte ^[QbgɑMAMf[^(0xaa) ҂ */
  ch = 0x55;
  char expected_ch = 0xaa;
  con->send(&ch, 1);

  int n = con->recv(&ch, 1, SCI_TIMEOUT);
  if (n == 0) {
    throw FZtatCtrlError("first connection: send 0x55 but no reply...");
  } else if (ch != expected_ch) {
    char actual[2] = { '0', '\0' };
    actual[0] = ch;
    throw FZtatCtrlError("first connection: send 0x55 and recieved ", &ch);
  }
}


void FZtatCtrl::sendWriteProgram(void) {
  
  /* ]vÕoCg ʁAʂ̏œ] */
  int code_size = sizeof(send_code)-1;
  char size_array[] = { code_size >> 8, code_size & 0xff };
  con->send(size_array, 2);
  
  /* ^[Qbg͎MMoCgxt@C˂đM */
  char *buf = new char[code_size];
  if (con->recv(buf, 2, SCI_TIMEOUT) == 0) {
    throw FZtatCtrlError("send write program size: timeout!");
  } else if (size_array[0] != buf[0] || size_array[1] != buf[1]) {
    throw FZtatCtrlError("send write program size: reply error!");
  }
  
  /* zXg̓oCgf[^Ƃē]vO𑗐M */
  con->send(send_code, code_size);
  con->recv(buf, code_size, SCI_TIMEOUT);

  /* tbVROMA^[Qbg 0xaa 𑗐M(s̏ꍇ 0xff) */
  if (con->recv(buf, 1, SCI_TIMEOUT) == 0) {
    throw FZtatCtrlError("send wirte code: not recieved return code 0xaa");
  } else if (buf[0] != (char)0xaa) {
    char actual[2] = { '0', '\0' };
    actual[0] = buf[0];
    throw FZtatCtrlError("send wirte code: recieved return code 0x", actual);
  }
  delete buf;
}


void FZtatCtrl::writeSetup(void) {
  con = new SerialDevice();
  if (con->connect(port.c_str(), SETUP_BAUDRATE) < 0) {
    throw FZtatCtrlError(port.c_str(), "device open fail");
  }
  
#ifndef SINGLE_DEBUG
  setupBaudrate();
  sendWriteProgram();
  con->setBaudrate(WRITE_BAUDRATE);
  con->send("@", 1);

  /*
  char ch;
  con->recv(&ch, 1, SCI_TIMEOUT);
  if (ch != '.') {
    throw FZtatCtrlError("after baudrate change: response timeout");
  }
  */
#endif
}


int FZtatCtrl::readSformat(srec_t *srec) {
  int line_cnt = 0;
  char line[49];
  int n = fscanf(fd, "%s\r\n", line);
  ++line_cnt;
  if (n == EOF || n == 0) {
    return -1;
  }
  if (parseSFormat(line, srec) < 0) {
    char message[256];
    sprintf(message, "srec file error at %d", line_cnt);
    throw FZtatCtrlError(message);
  }

  return srec->type;
}


// S-format file ǂݍ
int FZtatCtrl::loadSfile(void) {
  if (fd) {
    fclose(fd);
  }
  if (sfile.size() == 0) {
    throw FZtatCtrlError("file is not specified");
  }
  fd = fopen(sfile.c_str(), "r");
  if (!fd) {
    throw FZtatCtrlError(sfile.c_str(), "open fail");
  }
  line_cnt = 1;

  srec_t srec;
  int total = 0;
  while (readSformat(&srec) >= 0) {
    total += srec.data_size;
  }
  rewind(fd);
  line_cnt = 1;
  
  return total;
}


/* obt@̏ */
void FZtatCtrl::initBuffer(void) {
  for (int i = 0; i < 4; ++i) {
    buf.addr[i] = 0xff;
  }
  for (int i = 0; i < 32; ++i) {
    buf.data[i] = 0xff;
  }
  buffer_size = 0;
}


/* obt@ S-format ̗̈悪ꏊwԂ */
bool FZtatCtrl::isSameBuffer(srec_t *srec) {
  unsigned long address = srec->address & ~0x1f;
  unsigned long now_addr = 0;
  for (int i = 0; i < 4; ++i) {
    now_addr = (now_addr << 8) + (unsigned char)buf.addr[i];
  }
  return (now_addr == address);
}


/* obt@ɃAhXAf[^zu */
void FZtatCtrl::setBuffer(srec_t *srec) {
  unsigned long address = srec->address & ~0x1f;

  /* AhX̔zu */
  for (int i = 0; i < 4; ++i) {
    buf.addr[i] = (address >> 8*(3-i)) & 0xff;
  }

  /* f[^̔zu */
  for (int i = 0; i < srec->data_size; ++i) {
    buf.data[srec->address - address + i] = srec->byte_data[i];
  }

  buffer_size += srec->data_size;
}


/* Flush ROM ͈͓̔mF */
int FZtatCtrl::checkSendData(void) {
  unsigned long address = 0;
  int i;

  for (i = 0; i < 4; ++i) {
    address = (address << 8) + buf.addr[i];
  }
  if ((address < FLUSH_ROM_BEGIN) || (address >= FLUSH_ROM_END)) {
    return -1;
    //throw FZtatCtrlError("Warnning! write data is out of flush ROM");
  }
  return 0;
}


/* ^[Qbgɏރf[^𑗐MAGR[obN҂ */
void FZtatCtrl::sendWriteData(void) {
  char check;

  /* ރf[^̑M */
  if (checkSendData() < 0) { /* Flush ROM ͈͓̔mF */
    return;
  }
#ifndef SINGLE_DEBUG
  con->send((char*)buf.addr, 4);
  con->send((char*)buf.data, 32);
#endif
  
#ifndef SINGLE_DEBUG
  int n = con->recv(&check, 1, SCI_TOTAL_TIMEOUT);
#else
  int n = 1;
  check = '.';
#endif
  if (n == 0) {
    throw FZtatCtrlError("\ndata write: response timeout");
  }
  if (check == ':') {		// ݎs̉
    throw FZtatCtrlError("\ndata wirte: writing is failed!");
    
  } else if (check != '.') {	// sȉ
    char actual[2] = { '0', '\0' };
    actual[0] = check;
    throw FZtatCtrlError("\ndata write: response error, recieved ", actual);
  }
  
#ifndef SINGLE_DEBUG
  /* ^[Qbg̃GR[obN`FbN */
  data_set_t echoback_check;
  if (con->recv((char*)echoback_check.addr, 4, SCI_TIMEOUT) == 0 ||
      con->recv((char*)echoback_check.data, 32, SCI_TIMEOUT) == 0) {
    /* GR[obNf[^̃^CAEg */

    throw FZtatCtrlError("\nsend write data: echo back timeout!");
  }
  
  if (strncmp((const char*)buf.addr, (const char*)echoback_check.addr, 4)|| 
      strncmp((const char*)buf.data, (const char*)echoback_check.data, 32)){
    throw FZtatCtrlError("\nsend write data: echo back fail.");
  }
#endif
}


int FZtatCtrl::writeLine(void) {
  srec_t srec;
  int send_size = 0;

  while (1) {
    int type = readSformat(&srec);
    if (type == 0) {
      continue;
    }
    if (type < 0) {
      // obt@̃f[^łI
      sendWriteData();
      return -1;
    }
    
    if (type >= 1 && type <= 3) {
      
      // \ׂ
      if (!isSameBuffer(&srec)) {
	sendWriteData();
	send_size = buffer_size;
	initBuffer();
	setBuffer(&srec);
	break;
      } else {
	setBuffer(&srec);
      }
    }
  }

  return send_size;
}
