#ifndef F_ZTAT_CTRL_H
#define F_ZTAT_CTRL_H

/*
  F-ZTAT ւ̏ݐ
  Satofumi KAMIMURA
  $Id$
*/

#include "serialDevice.h"
#include "sformat.h"
#include <stdio.h>
#include <string>
#include <exception>


#ifdef _MBCS
class FZtatCtrlError
#else
class FZtatCtrlError : public std::exception
#endif
{
  enum { ERROR_MESSAGE_BUFFER_SIZE = 255 };
  char error_message_buffer[ERROR_MESSAGE_BUFFER_SIZE];
  
public:
  FZtatCtrlError(const char *device, const char *message) {
    sprintf(error_message_buffer, "%s : %s", device, message);
  }
  
  FZtatCtrlError(const char *message) {
    sprintf(error_message_buffer, "%s", message);
  }

  virtual const char* what(void) const throw() {
    return error_message_buffer;
  }
};


class FZtatCtrl {
  enum {
    SETUP_BAUDRATE = 9600,
    WRITE_BAUDRATE = 38400,
    SCI_TIMEOUT = 300,
    SCI_TOTAL_TIMEOUT = SCI_TIMEOUT * 3,

    FLUSH_ROM_BEGIN = 0x00000000,
    FLUSH_ROM_END = 0x0003ffff,
  };

  typedef struct {
    unsigned char addr[4];
    unsigned char data[32];
  } data_set_t;
  
  int sendSize;
  std::string sfile;
  std::string port;
  ConnectionDevice *con;
  FILE *fd;
  int line_cnt;
  data_set_t buf;
  int buffer_size;
  
  void setupBaudrate(void);
  void sendWriteProgram(void);
  int readSformat(srec_t *srec);
  void initBuffer(void);
  bool isSameBuffer(srec_t *srec);
  void setBuffer(srec_t *srec);
  int checkSendData(void);
  void sendWriteData(void);
  
public:
  FZtatCtrl(const char *sFormatFile, const char *serialPort);
  ~FZtatCtrl(void);
  void writeSetup(void);
  int loadSfile(void);
  int writeLine(void);
};

#endif /* !F_ZTAT_CTRL_H */
