#include "optionHandler.h"
#include <getopt.h>
#include <stdio.h>


void OptionHandler::print_help(void) {
  printf("Flush writer for SH7045F using TTY.\n"
	 "Usage:\n\tsh7045writer [options] <upload file>\n"
	 "\n"
	 "options:\n"
	 "  -h, --help\tPrint this message.\n"
	 "  --tty=TTY\tUse TTY for input/output by the program. (default %s)\n"
	 "  -b BAUDRATE\tSet serial port baud rate. (default %ld bps)\n",
	 tty_name.c_str(), baudrate);
  exit(1);
}


std::string OptionHandler::get_targetFile(void) {
  return target_file;
}


std::string OptionHandler::get_tty(void) {
  return tty_name;
}


long OptionHandler::get_baudrate(void) {
  return baudrate;
}


OptionHandler::OptionHandler(int argc, char *argv[])
#ifdef WIN32
  : target_file(""), tty_name("COM1"), baudrate(DEFAULT_BAUDRATE) {
#else
  : target_file(""), tty_name("/dev/ttyS0"), baudrate(DEFAULT_BAUDRATE) {
#endif
  
  static struct option long_options[] = {
    {"help", 0, 0, 0},
    {"h", 0, 0, 0},
    {"tty", 1, 0, 0},
    {"b", 1, 0, 0},
    {0, 0, 0, 0}
  };
  opterr = 0;

  int i, option_index = 0;
  while ((i = getopt_long(argc, argv, "",
			  long_options, &option_index)) != -1) {
    const char *optname = long_options[option_index].name;
    switch (i) {
    case 0:
      if (!strcmp("help", optname) || !strcmp("h", optname)) {
	// help
	print_help();

      } else if (!strcmp("tty", optname)) {
	// tty_name
	tty_name = optarg;

      } else if (!strcmp("b", optname)) {
	// baudrate
	baudrate = atoi(optarg);
      }
      break;

    default:
      print_help();
    }
  }
  if ((argc - optind) != 1) {
    return;
  }
  target_file = argv[optind];
}
